# Generated by Django 3.2.12 on 2022-03-02 22:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0010_auto_20220131_1539'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='display_name',
            field=models.CharField(blank=True, default='', help_text='A configuration nickname.', max_length=255),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='display_name',
            field=models.CharField(blank=True, default='', help_text='A configuration nickname.', max_length=255),
        ),
    ]
