# Generated by Django 3.2.15 on 2022-09-29 19:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0163_auto_20220928_1611'),
        ('integrated_channel', '0021_remove_contentmetadataitemtransmission_api_response_body'),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name='contentmetadataitemtransmission',
            index_together={('enterprise_customer', 'integrated_channel_code', 'plugin_configuration_id', 'content_id')},
        ),
    ]
