# Generated by Django 2.2.24 on 2021-12-03 18:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0150_invite_key_expiry_required'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomerinvitekey',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Specifies if the key is active. Once deactivated, the key is no longer valid and cannot be reactivated.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomerinvitekey',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Specifies if the key is active. Once deactivated, the key is no longer valid and cannot be reactivated.'),
        ),
    ]
