# Generated by Django 3.2.16 on 2022-12-20 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0021_move_and_recrete_completed_timestamp'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='grade_assignment_name',
            field=models.CharField(default='(edX integration) Final Grade', help_text='The name for the grade assigment created for the grade integration.', max_length=255, verbose_name='Grade Assignment Name'),
        ),
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='grade_scale',
            field=models.IntegerField(default=100, help_text='The maximum grade points for the courses. Default: 100', verbose_name='Grade Scale'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='grade_assignment_name',
            field=models.CharField(default='(edX integration) Final Grade', help_text='The name for the grade assigment created for the grade integration.', max_length=255, verbose_name='Grade Assignment Name'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='grade_scale',
            field=models.IntegerField(default=100, help_text='The maximum grade points for the courses. Default: 100', verbose_name='Grade Scale'),
        ),
    ]
